package de.kolbware.springOsgiUtil.context;

import javax.servlet.ServletContext;

import org.osgi.framework.BundleContext;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.osgi.context.support.BundleDelegatingClassLoader;
import org.springframework.osgi.context.support.LocalBundleContext;
import org.springframework.web.context.ContextLoader;
import org.springframework.web.context.WebApplicationContext;

/**
 * A {@link ContextLoader} which creates a {@link WrappedApplicationContext} for
 * the given {@link BundleContext}
 * 
 * @author Bernd Kolb
 * 
 */
public class BundleContextLoader extends ContextLoader {

	private final ApplicationContext appContext;
	private BundleContext ctx;

	public BundleContextLoader(ApplicationContext appContext, BundleContext ctx) {
		this.appContext = appContext;
		this.ctx = ctx;
	}

	@Override
	protected ApplicationContext loadParentContext(ServletContext servletContext)
			throws BeansException {
		return appContext;
	}

	/**
	 * @return a {@link WrappedApplicationContext}
	 * @see org.springframework.web.context.ContextLoader#createWebApplicationContext(javax.servlet.ServletContext,
	 *      org.springframework.context.ApplicationContext)
	 */
	protected WebApplicationContext createWebApplicationContext(
			ServletContext servletContext, ApplicationContext parent)
			throws BeansException {
		ClassLoader contextClassLoader = Thread.currentThread()
				.getContextClassLoader();
		try {
			ClassLoader cl = BundleDelegatingClassLoader
					.createBundleClassLoaderFor(ctx.getBundle(), getClass()
							.getClassLoader());
			Thread.currentThread().setContextClassLoader(cl);
			LocalBundleContext.setContext(ctx);
			return new WrappedApplicationContext(parent, servletContext);
		} finally {
			Thread.currentThread().setContextClassLoader(contextClassLoader);
		}
	}

}
