package de.kolbware.springOsgiUtil.context;

import org.osgi.framework.BundleContext;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.ContextLoader;
import org.springframework.web.context.ContextLoaderServlet;

/**
 * A customized version of the {@link ContextLoaderServlet} for OSGi. Replaces
 * the default implementation of
 * {@link ContextLoaderServlet#createContextLoader}
 * 
 * @author Bernd Kolb
 * 
 */
public class BundleContextLoaderServlet extends ContextLoaderServlet {

	private final ApplicationContext appContext;
	private BundleContext ctx;

	/**
	 * @param appContext
	 *            {@link ApplicationContext} for the containing bundle
	 * @param ctx
	 *            {@link BundleContext} of the current bundle
	 */
	public BundleContextLoaderServlet(ApplicationContext appContext,
			BundleContext ctx) {
		this.appContext = appContext;
		this.ctx = ctx;
	}

	/**
	 * @return a {@link BundleContextLoader}
	 * @see org.springframework.web.context.ContextLoaderServlet#createContextLoader()
	 */
	@Override
	protected ContextLoader createContextLoader() {
		return new BundleContextLoader(appContext, ctx);
	}

}
