package de.kolbware.springOsgiUtil.context;

import org.osgi.framework.BundleContext;
import org.springframework.beans.BeansException;
import org.springframework.osgi.context.support.BundleDelegatingClassLoader;
import org.springframework.osgi.context.support.LocalBundleContext;
import org.springframework.util.StringUtils;
import org.springframework.web.context.ConfigurableWebApplicationContext;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.servlet.DispatcherServlet;
import org.springframework.web.servlet.FrameworkServlet;

/**
 * A custimized {@link DispatcherServlet} for the usage within OSGi. Replaces
 * the default implementation of
 * {@link FrameworkServlet#createWebApplicationContext(WebApplicationContext)}
 * 
 * @author Bernd Kolb
 * 
 */
public class OSGiDispatcherServlet extends DispatcherServlet {

	private BundleContext ctx;

	public OSGiDispatcherServlet(BundleContext ctx) {
		this.ctx = ctx;
	}

	/**
	 * creates a new {@link OSGiXmlWebApplicationContext} and initializes it
	 * 
	 * @param parent
	 *            the parents {@link WebApplicationContext}
	 * @see org.springframework.web.servlet.FrameworkServlet#createWebApplicationContext(org.springframework.web.context.WebApplicationContext)
	 */
	@Override
	protected WebApplicationContext createWebApplicationContext(
			WebApplicationContext parent) throws BeansException {
		ClassLoader contextClassLoader = Thread.currentThread()
				.getContextClassLoader();
		try {
			ClassLoader cl = BundleDelegatingClassLoader
					.createBundleClassLoaderFor(ctx.getBundle(), getClass()
							.getClassLoader());
			Thread.currentThread().setContextClassLoader(cl);
			LocalBundleContext.setContext(ctx);

			ConfigurableWebApplicationContext wac = new OSGiXmlWebApplicationContext(
					ctx);
			wac.setParent(parent);
			wac.setServletContext(getServletContext());
			wac.setServletConfig(getServletConfig());
			wac.setNamespace(getNamespace());
			if (getContextConfigLocation() != null) {
				wac
						.setConfigLocations(StringUtils
								.tokenizeToStringArray(
										getContextConfigLocation(),
										ConfigurableWebApplicationContext.CONFIG_LOCATION_DELIMITERS));
			}
			wac.addApplicationListener(this);
			wac.refresh();
			return wac;
		} finally {
			Thread.currentThread().setContextClassLoader(contextClassLoader);
		}
	}

}
