package de.kolbware.springOsgiUtil.context;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.beans.factory.config.Scope;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.osgi.context.BundleContextAware;
import org.springframework.osgi.context.OsgiBundleScope;
import org.springframework.osgi.context.support.BundleContextAwareProcessor;
import org.springframework.osgi.io.OsgiBundleResource;
import org.springframework.osgi.io.OsgiBundleResourcePatternResolver;
import org.springframework.util.Assert;
import org.springframework.web.context.ConfigurableWebApplicationContext;
import org.springframework.web.context.support.XmlWebApplicationContext;

/**
 * A web application context which can find {@link Resource}s relative to a
 * {@link Bundle}
 * 
 * @author Bernd Kolb
 * 
 */
public class OSGiXmlWebApplicationContext extends XmlWebApplicationContext
		implements ConfigurableWebApplicationContext {

	private final BundleContext ctx;

	public OSGiXmlWebApplicationContext(BundleContext ctx) {
		this.ctx = ctx;
	}

	protected ResourcePatternResolver getResourcePatternResolver() {
		return new OsgiBundleResourcePatternResolver(this);
	}

	public Resource getResource(String location) {
		Assert.notNull(location, "Location must not be null");
		if (location.startsWith(CLASSPATH_URL_PREFIX)) {
			return new ClassPathResource(location
					.substring(CLASSPATH_URL_PREFIX.length()), getClassLoader());
		} else {
			return new OsgiBundleResource(ctx.getBundle(), location);
		}
	}

	@Override
	protected void postProcessBeanFactory(
			ConfigurableListableBeanFactory beanFactory) {
		super.postProcessBeanFactory(beanFactory);

		// copied from the spring osgi implementation

		beanFactory.addBeanPostProcessor(new BundleContextAwareProcessor(
				this.ctx));
		beanFactory.ignoreDependencyInterface(BundleContextAware.class);

		// register 'bundle' scope (but make sure we clean it, in case of a
		// refresh)
		cleanOsgiBundleScope(beanFactory);
		beanFactory.registerScope(OsgiBundleScope.SCOPE_NAME,
				new OsgiBundleScope());

		// endcopy
	}

	/*
	 * copied from the spring osgi implementation
	 */
	protected void cleanOsgiBundleScope(
			ConfigurableListableBeanFactory beanFactory) {
		Scope scope = beanFactory
				.getRegisteredScope(OsgiBundleScope.SCOPE_NAME);
		if (scope != null && scope instanceof OsgiBundleScope)
			((OsgiBundleScope) scope).destroy();

	}

}
