package de.kolbware.springOsgiUtil.http;

import java.util.ArrayList;
import java.util.List;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.http.HttpService;
import org.osgi.util.tracker.ServiceTracker;
import org.springframework.context.ApplicationContext;

/**
 * A {@link ServiceTracker} implementation which waits until the
 * {@link ApplicationContext} is available and gives the implementor a
 * {@link HttpService} and the {@link ApplicationContext} to register resources
 * and servlets on this service.
 * 
 * @author Bernd Kolb
 * 
 */
public abstract class AppContextAwareHttpServiceTracker extends ServiceTracker {

	private ApplicationContext applicationContext;
	private List<HttpService> httpServices = new ArrayList<HttpService>();

	public AppContextAwareHttpServiceTracker(BundleContext context) {
		super(context, HttpService.class.getName(), null);
	}

	public void setAppContext(ApplicationContext applicationContext) {
		this.applicationContext = applicationContext;
		checkAndCall();
	}

	private void checkAndCall() {
		if (applicationContext == null || httpServices.size() == 0) {
			return;
		}
		try {
			for (HttpService httpService : httpServices) {
				addServiceInternal(applicationContext, httpService);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	@Override
	public Object addingService(ServiceReference reference) {
		Object service = super.addingService(reference);
		if (service instanceof HttpService) {
			addHttpService((HttpService) service);
		}
		return service;
	}

	private void addHttpService(HttpService addingService) {
		this.httpServices.add(addingService);
		checkAndCall();

	}

	/**
	 * @param appContext
	 *            the {@link ApplicationContext} for the bundle in which the
	 *            service tracker has been initialized
	 * @param httpService
	 *            Reference to service being added to this
	 *            <code>ServiceTracker</code> object.
	 * @throws Exception
	 */
	protected abstract void addServiceInternal(ApplicationContext appContext,
			HttpService httpService) throws Exception;

}
