package de.kolbware.springOsgiUtil.http;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.BundleListener;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.osgi.context.BundleContextAware;

/**
 * A utility bean which could be used to register http services. To use it write
 * something like the following code <br>
 * <code>
 * &lt;bean id=&quot;serviceRegistrator&quot; class=&quot;de.kolbware.springOsgiUtil.http.ServiceRegistrator&quot;&gt;<br>
 * &nbsp;&nbsp;&nbsp;&lt;constructor-arg value=&quot;de.kolbware.jpetstore.internal.web.HttpServiceTracker&quot;/&gt;<br>
 * &lt;/bean&gt;
 * </code>
 * 
 * @author Bernd Kolb
 * 
 */
public class ServiceRegistrator implements ApplicationContextAware,
		BundleContextAware, BundleListener {

	private ApplicationContext applicationContext;
	private BundleContext context;
	private Class<? extends AppContextAwareHttpServiceTracker> httpServiceTrackerClass;
	private AppContextAwareHttpServiceTracker httpServiceTracker;

	/**
	 * @param httpServiceTrackerClass
	 *            a class extending {@link AppContextAwareHttpServiceTracker}
	 */
	public ServiceRegistrator(
			Class<? extends AppContextAwareHttpServiceTracker> httpServiceTrackerClass) {
		this.httpServiceTrackerClass = httpServiceTrackerClass;
	}

	public void setApplicationContext(ApplicationContext applicationContext)
			throws BeansException {
		this.applicationContext = applicationContext;
		if (httpServiceTracker != null) {
			httpServiceTracker.setAppContext(applicationContext);
		}

	}

	public void setBundleContext(BundleContext context) {
		try {
			httpServiceTracker = httpServiceTrackerClass.getConstructor(
					BundleContext.class).newInstance(context);
			httpServiceTracker.open();
			context.addBundleListener(this);
			if (applicationContext != null) {
				httpServiceTracker.setAppContext(applicationContext);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void bundleChanged(BundleEvent event) {
		if (event.getBundle() == context.getBundle()
				&& event.getType() == Bundle.STOPPING) {
			httpServiceTracker.close();
		}
	}

}
